@@ -1266,7 +1266,7 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 {
     int ruby_error, old_ruby_quiet;
     VALUE err;
-    char* process_options_argv[] = { "ruby", "-enil", NULL };
+    char* ruby_options_argv[] = { "ruby", "-enil", NULL };
     char *weechat_ruby_code = {
         "$stdout = WeechatOutputs\n"
         "$stderr = WeechatOutputs\n"
@@ -1365,7 +1365,7 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 
     ruby_init ();
 
-    ruby_process_options (2, process_options_argv);
+    ruby_options (2, ruby_options_argv);
 
     /* redirect stdin and stdout */
     ruby_mWeechatOutputs = rb_define_module ("WeechatOutputs");
